"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var React = require("react");
var ReactDOM = require("react-dom");
var _ = require("underscore");
var icon_1 = require('./../display/icon');
var grid_1 = require('./../layout/grid');
var button_1 = require('./../interaction/button');
var DropdownSelect = (function (_super) {
    __extends(DropdownSelect, _super);
    function DropdownSelect() {
        _super.call(this);
        this.id = "dropdown-select-" + Math.random();
        this.minimumLength = 1;
        this.state = { filteredOptions: [], query: "", open: false, selectedValue: null, selectedIndex: 0 };
    }
    DropdownSelect.prototype.filter = function (query) {
        var _this = this;
        var q = query.toLowerCase();
        if (query.length <= this.minimumLength) {
            this.setState({ filteredOptions: this.props.options, query: query }, function () { return _this.constrainIndex(); });
        }
        else {
            this.setState({ filteredOptions: _.reject(this.props.options, function (o) { return o.label.toLowerCase().indexOf(q) === -1; }), query: query }, function () { return _this.constrainIndex(); });
            ;
        }
    };
    DropdownSelect.prototype.focusInput = function () {
        var _this = this;
        if (!this.state.open) {
            this.setState({ open: true }, function () {
                document.getElementById(_this.id).focus();
                document.addEventListener("click", _this, false);
            });
        }
        else {
            this.setState({ open: false, query: "", filteredOptions: this.props.options });
            document.removeEventListener("click", this, false);
        }
    };
    DropdownSelect.prototype.handleEvent = function (e) {
        if (ReactDOM.findDOMNode(this).contains(e.target)) {
            return;
        }
        this.setState({ open: false, query: "", filteredOptions: this.props.options });
        document.removeEventListener("click", this, false);
    };
    DropdownSelect.prototype.componentWillMount = function () {
        this.setState({ filteredOptions: this.props.options });
    };
    DropdownSelect.prototype.checkKey = function (e) {
        var currentIndex = this.state.selectedIndex;
        if (e.keyCode === 40) {
            this.setState({ selectedIndex: Math.min(this.state.selectedIndex + 1, this.state.filteredOptions.length - 1) });
            e.preventDefault();
            return false;
        }
        if (e.keyCode === 38) {
            this.setState({ selectedIndex: Math.max(this.state.selectedIndex - 1, 0) });
            e.preventDefault();
            return false;
        }
        if (e.keyCode === 13) {
            this.setState({ selectedValue: this.state.filteredOptions[this.state.selectedIndex], open: false, query: "", filteredOptions: this.props.options });
            document.removeEventListener("click", this, false);
            e.preventDefault();
            return false;
        }
    };
    DropdownSelect.prototype.constrainIndex = function () {
        if (this.state.selectedIndex > this.state.filteredOptions.length - 1) {
            this.setState({ selectedIndex: Math.max(this.state.filteredOptions.length - 1, 0) });
        }
    };
    DropdownSelect.prototype.buttonClick = function () {
        this.setState({ selectedValue: this.state.filteredOptions[this.state.selectedIndex], open: false, query: "", filteredOptions: this.props.options });
        document.removeEventListener("click", this, false);
    };
    DropdownSelect.prototype.render = function () {
        var _this = this;
        return (React.createElement("div", {className: "dropdown-select"}, React.createElement(grid_1.Grid, {className: "dropdown-value-display"}, React.createElement(grid_1.Row, null, React.createElement(grid_1.Col, {onClick: function () { return _this.focusInput(); }}, this.state.selectedValue ? React.createElement("div", null, this.state.selectedValue.label) : React.createElement("div", {className: "placeholder"}, this.props.placeholder)), React.createElement(grid_1.Col, {fixed: true, className: "clear-selected p-right-xsmall", onClick: function () { return _this.setState({ selectedValue: null, open: false, query: "", filteredOptions: _this.props.options }); }}, React.createElement(icon_1.Icon, {icon: icon_1.Icon.Icomoon.cross})), React.createElement(grid_1.Col, {fixed: true}, React.createElement(button_1.Button, {text: "Go", className: "bg-positive", onClick: function () { return _this.buttonClick(); }})))), this.state.open &&
            React.createElement("div", {className: "dropdown-select-list-wrapper"}, React.createElement("input", {type: "text", id: this.id, onKeyUp: function (e) { return _this.checkKey(e); }, defaultValue: this.props.value, onChange: function (e) { return _this.filter(e.target.value); }, placeholder: "start typing to filter results..."}), React.createElement("div", {className: "dropdown-select-list"}, this.state.filteredOptions.map(function (o, i) { return React.createElement("div", {key: "dd-item-" + i, className: "dd-list-item" + (i === _this.state.selectedIndex ? ' selected' : ''), onClick: function () { return _this.setState({ selectedValue: o, open: false, query: "", filteredOptions: _this.props.options, selectedIndex: _this.props.options.indexOf(o) }); }}, o.label); }), this.state.filteredOptions.length === 0 && React.createElement("div", null, "No results...")))));
    };
    return DropdownSelect;
}(React.Component));
exports.DropdownSelect = DropdownSelect;
